//
// Cyrano Chess engine
//
// Copyright (C) 2007  Harald JOHNSEN
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
//
//


#ifndef _EVAL_HXX
#define _EVAL_HXX

// Pawn structure hash element
// The pawn hash contains informations and *static* eval of the pawn structure
// The real value of the pawn structure depends of the complete position (and side
// to move), this value is not integrated in the val tag
// The eval() function will use the pawn structure information to recompute the
// real value depending on the phase of the game

//#define KEYSQUARE

typedef struct {
    int distance[2][64];
    int square;
    int winningSide;
} squareDist;

typedef	struct tagHASHP {
	HASHK	hashk;				// Pawn key (other pieces not in here).
//	int	val;					// *static* value from the perspective of white.
    struct tagStructure {
        bitboard influence;     // squares that could be attacked by a pawn
        bitboard candidate;     // candidate pawn
        bitboard passed;        // not blocked and not within influence(xside)
        bitboard location;      // position of pawns
        bitboard pawnatk;       // squares attack by pawns
        int     val;
        U8 c_pawn[8];           // number of pawn per file
        U8 c_doubled;           // number of doubled pawn in the position
        U8 c_islands;           // number of pawn islands
        U8 c_backward;          // count of backward pawns
        U8 c_connected;         // count of connected pawns
    } side[2];                  // this is for each side
#ifdef KEYSQUARE
    squareDist  keySquare[3];
#endif
    bool    pawnon6or7;
    U8      c_center_pawns;     // number of pawns in the central square
}	HASHPawn, * PHASHPawn;

typedef struct {
    // counters
    int c_move;         // non capture + capture + defended material
    int c_king;
    int c_offense;      // number of square in the ennemy camp where we can go
    int c_bishop;       // mobility of bishops
    bitboard    attacks;
} _dynamicVal;

extern int pathLength(U64 sq1, U64 sq2, U64 path);
extern void EvalPawns(const CON *pcon, const STE *pste, PHASHPawn & phashp);
extern int const remapSq[2][64];
extern const int exchangePieceValue[16];
extern int	const staticPosVal[6+2][64];

extern void computeMobility(CON *pcon, PSTE pste, int side, bitboard xpawns, _dynamicVal & dynamicVal);
extern int see(PCON pcon, const STE *pste, int from, int to);
extern void initEG(PCON pcon, PSTE pste);
bool KPK(PCON pcon, PSTE pste, int co, int & val);
bool KBNK(PCON pcon, PSTE pste, int co, int & val);


#endif // _EVAL_HXX